package edu.kit.ipd.descartes.ql.connector.nop;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.ecore.util.EcoreUtil;

import edu.kit.ipd.descartes.ql.connector.ConnectorProvider;
import edu.kit.ipd.descartes.ql.connector.ModelStructureQueryConnector;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DomainFactory;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Probe;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Resource;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Service;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

public class ModelStructureQueryConnectorImpl extends AbstractNopQueryConnector implements ModelStructureQueryConnector {

	private final DomainFactory domainFactory = DomainFactory.eINSTANCE;

	public ModelStructureQueryConnectorImpl() {
		log.info(ModelStructureQueryConnector.class.getName() + " initialized");
		log.info("Handling requests for performance meta-model family "
				+ getBundleProperty(ConnectorProvider.FAMILY_KEY));
	}

	private boolean isValidEntitiesRequest(final EntityMapping request) {
		return isValidRequest(request);
	}

	private boolean isValidProbesRequest(final EntityMapping request) {
		// TODO: Beautify
		return true;
	}

	private boolean isValidRequest(final EntityMapping request) {
		final List<Service> services = request.getServices();
		final List<Resource> resources = request.getResources();

		if (services.isEmpty() && resources.isEmpty() && !request.getModelLocation().isEmpty()) {
			log.info("Request validated");
			return true;
		}

		log.warn("Invalid request");
		return false;
	}

	public EntityMapping requestDoF(final EntityMapping request) {
		return null;
	}

	public EntityMapping requestEntities(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final List<Service> services = response.getServices();
		final List<Resource> resources = response.getResources();

		log.info("LIST ENTITIES requested");

		// The incoming request shall not contain a listing
		if (!isValidEntitiesRequest(request)) {
			return null;
		}

		for (int i = 0; i < NUMBER_OF_ELEMENTS; i++) {
			services.add(createService(domainFactory, i));
			resources.add(createResource(domainFactory, i));
		}

		return response;
	}

	public EntityMapping requestProbes(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final ArrayList<Entity> entities = new ArrayList<Entity>();

		entities.addAll(response.getServices());
		entities.addAll(response.getResources());

		log.info("LIST METRICS requested");

		// The incoming request shall not contain a listing
		if (!isValidProbesRequest(request)) {
			return null;
		}

		for (final Entity entity : entities) {
			for (int j = 0; j < NUMBER_OF_ELEMENTS; j++) {
				final Probe aProbe = createProbe(domainFactory, j);
				entity.getProbes().add(aProbe);
			}
		}

		return response;
	}

	public boolean supportsDoFRequests() {
		return false;
	}

	public boolean supportsEntityRequests() {
		return true;
	}

	public boolean supportsProbeRequests() {
		return true;
	}

}
