package edu.kit.ipd.descartes.ql.connector.nop;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;

import org.eclipse.emf.ecore.util.EcoreUtil;

import edu.kit.ipd.descartes.ql.connector.PerformanceMetricsQueryConnector;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DecimalResult;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DoF;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DomainFactory;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Probe;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

public class PerformanceMetricsQueryConnectorImpl extends AbstractNopQueryConnector implements
		PerformanceMetricsQueryConnector {
	ArrayList<String> constraintNames = new ArrayList<String>();
	ArrayList<String> doFConstraintNames = new ArrayList<String>();
	ArrayList<String> explorationStrategyNames = new ArrayList<String>();
	private String constraint = null;
	private String doFConstraint = null;
	private String exploratationStrategy = null;
	private Properties explorationStrategyProperties;
	private final DomainFactory domainFactory = DomainFactory.eINSTANCE;

	public PerformanceMetricsQueryConnectorImpl() {
		super();

		loadConstraintNames();
		loadDoFConstraintNames();
		loadExplorationStrategyNames();
	}

	public String getConstraint() {
		return constraint;
	}

	public String getDoFConstraint() {
		return doFConstraint;
	}

	public String getDoFExplorationStrategy() {
		return exploratationStrategy;
	}

	public Properties getDoFExplorationStrategyProperties() {
		return explorationStrategyProperties;
	}

	public List<String> getSupportedConstraintNames() {
		return constraintNames;
	}

	public List<String> getSupportedDoFConstraintNames() {
		return doFConstraintNames;
	}

	public List<String> getSupportedExplorationStrategyNames() {
		return explorationStrategyNames;
	}

	public List<String> getSupportedTimeUnits() {
		// TODO Auto-generated method stub
		return null;
	}

	private void loadConstraintNames() {
		constraintNames.add("DEFAULT");
		constraintNames.add("FAST");
		constraintNames.add("ACCURATE");
		constraintNames.add("TIME_1s");
		constraintNames.add("TIME_5s");
		constraintNames.add("ACCURACY_95");
		constraintNames.add("ACCURACY_99");
	}

	private void loadDoFConstraintNames() {
		doFConstraintNames.add("DEFAULT");
		doFConstraintNames.add("FAST");
		doFConstraintNames.add("ACCURATE");
		doFConstraintNames.add("TIME_1s");
		doFConstraintNames.add("TIME_5s");
		doFConstraintNames.add("ACCURACY_95");
		doFConstraintNames.add("ACCURACY_99");
	}

	private void loadExplorationStrategyNames() {
		explorationStrategyNames.add("FULL");
		explorationStrategyNames.add("RANDOM");
	}

	public List<EntityMapping> requestDoFMetrics(final EntityMapping request) {
		final ArrayList<EntityMapping> results = new ArrayList<EntityMapping>();

		for (@SuppressWarnings("unused")
		final DoF doF : request.getDoF()) {
			results.add(requestMetrics(request));
		}

		return results;
	}

	public EntityMapping requestMetrics(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final ArrayList<Entity> entities = new ArrayList<Entity>();
		entities.addAll(response.getResources());
		entities.addAll(response.getServices());

		for (final Entity entity : entities) {
			final ArrayList<DecimalResult> results = new ArrayList<DecimalResult>();

			for (final Probe probe : entity.getProbes()) {
				final DecimalResult result = domainFactory.createDecimalResult();

				result.setMetricName(probe.getMetricName());
				result.setValid(true);
				result.setAccuracy(new BigDecimal(0.9995f));
				result.setValue(new BigDecimal(1.234f));

				results.add(result);
			}

			entity.getProbes().clear();
			entity.getProbes().addAll(results);
		}

		return response;
	}

	public void setConstraint(final String constraint) {
		// Although being not relevant for results, the value is stored and
		// returned by the getter, e.g. for regression testing.
		if (constraintNames.contains(constraint)) {
			this.constraint = constraint;
		} else {
			this.constraint = null;
		}
	}

	public void setDoFConstraint(final String doFConstraint) {
		if (doFConstraintNames.contains(doFConstraint)) {
			this.doFConstraint = doFConstraint;
		} else {
			this.doFConstraint = null;
		}
	}

	public void setDoFExplorationStrategy(final String strategy, final Properties properties) {
		if (explorationStrategyNames.contains(strategy)) {
			exploratationStrategy = strategy;
		} else {
			exploratationStrategy = null;
		}

		explorationStrategyProperties = properties;
	}

	public void setObservationPeriod(final Date start, final Date end, final long sampleSize) {
		// TODO Auto-generated method stub

	}

	public void setObservationPeriod(final long amount, final String unit, final long sampleSize) {
		// TODO Auto-generated method stub

	}

	public void setObservationReference(final String type, final String identifier, final long sampleSize) {

	}

	public boolean supportsConstraintRequests() {
		return true;
	}

	public boolean supportsDoFRequests() {
		return true;
	}

	public boolean supportsMetricRequests() {
		return true;
	}

	public boolean supportsObservationPeriods() {
		return false;
	}

	public void reset() {
		// Nothing to do here
	}

}
