package edu.kit.ipd.descartes.ql.connector.pdr;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EcoreUtil;

import edu.kit.ipd.descartes.ql.connector.AbstractOSGiQueryConnector;
import edu.kit.ipd.descartes.ql.connector.ModelStructureQueryConnector;
import edu.kit.ipd.descartes.ql.connector.util.DQLLogger;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DomainFactory;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Probe;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Resource;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Service;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;
import edu.kit.ipd.descartes.ql.models.repository.Observation;
import edu.kit.ipd.descartes.ql.models.repository.ObservationRepository;

public class ModelStructureQueryConnectorImpl extends AbstractOSGiQueryConnector implements
		ModelStructureQueryConnector {
	private final Logger log = DQLLogger.getLogger(this.getClass().getName());
	private final DomainFactory domainFactory = DomainFactory.eINSTANCE;

	public EntityMapping requestDoF(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final String modelLocation = RepositoryHelper.qualifyModelLocation(request.getModelLocation());

		// open model
		final ObservationRepository repository = RepositoryHelper.loadModelInstance(modelLocation);

		// iterate
		final Observation latestObservation = repository.getLatestObservation();
		final EntityMapping latest = latestObservation.getObservation();

		// copy collections
		response.getDoF().addAll(EcoreUtil.copyAll(latest.getDoF()));

		return response;
	}

	public EntityMapping requestEntities(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final String modelLocation = RepositoryHelper.qualifyModelLocation(request.getModelLocation());

		// open model
		final ObservationRepository repository = RepositoryHelper.loadModelInstance(modelLocation);

		// iterate
		final Observation latestObservation = repository.getLatestObservation();
		final EntityMapping latest = latestObservation.getObservation();
		final EList<Service> responseServices = response.getServices();
		final EList<Resource> responseResources = response.getResources();

		for (final Resource r : latest.getResources()) {
			final Resource copy = EcoreUtil.copy(r);

			copy.getProbes().clear();

			responseResources.add(copy);
		}

		for (final Service s : latest.getServices()) {
			final Service copy = EcoreUtil.copy(s);

			copy.getProbes().clear();
			responseServices.add(copy);
		}

		return response;
	}

	public EntityMapping requestProbes(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final String modelLocation = RepositoryHelper.qualifyModelLocation(request.getModelLocation());

		// open model
		final ObservationRepository repository = RepositoryHelper.loadModelInstance(modelLocation);

		// iterate
		final Observation latestObservation = repository.getLatestObservation();
		final EntityMapping latest = latestObservation.getObservation();
		final Map<String, Entity> latestEntityIndex = RepositoryHelper.buildEntityIndex(latest);
		final List<Entity> responseEntities = new LinkedList<Entity>();

		responseEntities.addAll(response.getResources());
		responseEntities.addAll(response.getServices());

		for (final Entity e : responseEntities) {
			final String identifier = e.getIdentifier();
			final Entity latestEntity = latestEntityIndex.get(identifier);

			if (latestEntity == null) {
				log.warn("No entity found for identifier " + identifier);
				continue;
			}

			for (final Probe latestProbe : latestEntity.getProbes()) {
				final Probe responseProbe = domainFactory.createProbe();
				final EList<Probe> probes = e.getProbes();

				responseProbe.setMetricName(latestProbe.getMetricName());
				probes.add(responseProbe);
			}
		}

		return response;
	}

	public boolean supportsDoFRequests() {
		return true;
	}

	public boolean supportsEntityRequests() {
		return true;
	}

	public boolean supportsProbeRequests() {
		return true;
	}

}
