package edu.kit.ipd.descartes.ql.connector.pdr;

import static edu.kit.ipd.descartes.ql.connector.pdr.RepositoryHelper.buildProbeIndex;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EcoreUtil;

import edu.kit.ipd.descartes.ql.connector.AbstractOSGiQueryConnector;
import edu.kit.ipd.descartes.ql.connector.PerformanceMetricsQueryConnector;
import edu.kit.ipd.descartes.ql.connector.util.DQLLogger;
import edu.kit.ipd.descartes.ql.core.engine.util.EntityMappingHelper;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DecimalResult;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Probe;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Resource;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Service;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;
import edu.kit.ipd.descartes.ql.models.repository.ObservableImpact;
import edu.kit.ipd.descartes.ql.models.repository.Observation;
import edu.kit.ipd.descartes.ql.models.repository.ObservationRepository;

public class PerformanceMetricsQueryConnectorImpl extends AbstractOSGiQueryConnector implements
		PerformanceMetricsQueryConnector {
	private static final String IMPACT_REFERENCE = "Impact";
	private static final String BEFORE_UNIT = "before";
	private final Logger log = DQLLogger.getLogger(this.getClass().getName());
	
	@SuppressWarnings("unused")	// Maybe later, if required
	private Date observationStartDate = null;
	@SuppressWarnings("unused")	// Maybe later, if required
	private Date observationEndDate = null;
	@SuppressWarnings("unused")	// Maybe later, if required
	private long observationSampleSize = 0;
	
	private boolean withObserve = false;
	private boolean framedObservation = false;
	private boolean observationByReference = false;
	private String observationUnit = null;
	private String observationIdentifier = null;
	private long observationAmountUnits = 0;

	private void buildImpactIndex(final Map<String, ObservableImpact> impactIndex, final EList<ObservableImpact> impacts) {
		for (final ObservableImpact i : impacts) {
			impactIndex.put(i.getIdentifier(), i);
		}
	}

	private EntityMapping calculateDifference(final EntityMapping after, final EntityMapping before) {
		final EntityMapping result = EcoreUtil.copy(after);
		final List<Entity> entities = new LinkedList<Entity>();
		final Map<String, Map<String, Probe>> probeIndex = EntityMappingHelper.buildProbeIndex(before);
		final EList<Resource> resources = result.getResources();
		final EList<Service> services = result.getServices();

		entities.addAll(resources);
		entities.addAll(services);

		for (final Entity e : entities) {
			final String identifier = e.getIdentifier();

			for (final Probe p : e.getProbes()) {
				if (!(p instanceof DecimalResult)) {
					log.warn(p + " in the 'after' mapping is no instance of DecimalResult. Skipping.");
					continue;
				}

				final String metricName = p.getMetricName();
				final DecimalResult probeResult = (DecimalResult) p;

				final Map<String, Probe> probeBeforeIndex = probeIndex.get(identifier);
				final Probe probeBefore = probeBeforeIndex.get(metricName);

				if (!(probeBefore instanceof DecimalResult)) {
					log.warn(p + " in the 'after' mapping is no instance of DecimalResult. Skipping.");
				}

				final DecimalResult probeBeforeResult = (DecimalResult) probeBefore;

				final BigDecimal resultAfter = probeResult.getValue();
				final BigDecimal resultBefore = probeBeforeResult.getValue();

				probeResult.setValue(resultAfter.subtract(resultBefore));
			}
		}

		return result;
	}

	private EntityMapping createInstanceFromImpact(final EntityMapping response, final ObservationRepository repository) {
		final EList<ObservableImpact> impacts = repository.getImpacts();
		final Map<String, ObservableImpact> impactIndex = new HashMap<String, ObservableImpact>(impacts.size());
		repository.getObservations();

		// Is the impact known?
		buildImpactIndex(impactIndex, impacts);
		final ObservableImpact impact = impactIndex.get(observationIdentifier);

		if (impact == null) {
			final String message = "Unknown impact referenced: " + observationIdentifier;

			log.error(message);
			throw new IllegalArgumentException(message);
		}

		// Resolve before and after mappings
		final EntityMapping before = impact.getBefore().getObservation();
		final EntityMapping after = impact.getAfter().getObservation();
		final EntityMapping result = calculateDifference(after, before);

		// TODO
		return result;
	}

	private void createResponseFromInstance(final EntityMapping response, final EntityMapping instance) {
		final Map<String, Entity> latestEntityIndex = RepositoryHelper.buildEntityIndex(instance);
		final List<Entity> responseEntities = new LinkedList<Entity>();

		responseEntities.addAll(response.getResources());
		responseEntities.addAll(response.getServices());

		for (final Entity e : responseEntities) {
			final String identifier = e.getIdentifier();
			final Entity latestEntity = latestEntityIndex.get(identifier);

			if (latestEntity == null) {
				log.warn("No entity found for identifier " + identifier);
				continue;
			}

			// Found entity, replace probes
			final Map<String, Probe> latestProbeIndex = buildProbeIndex(latestEntity);

			final List<Probe> requestedProbes = e.getProbes();
			final List<Probe> removal = new LinkedList<Probe>();
			final List<Probe> additional = new LinkedList<Probe>();

			for (final Probe requestedProbe : requestedProbes) {
				final String metricName = requestedProbe.getMetricName();
				final Probe latestProbe = latestProbeIndex.get(metricName);

				if (latestProbe == null) {
					log.warn("No result for entity " + e.getIdentifier() + " on metric " + metricName);
					continue;
				}

				// Found probe, replace it - outside of this loop as the
				// iterator does not like removing elements from list, while
				// iterating over the list. ;)
				removal.add(requestedProbe);
				additional.add(EcoreUtil.copy(latestProbe));
			}

			requestedProbes.removeAll(removal);
			requestedProbes.addAll(additional);
		}
	}

	public String getConstraint() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getDoFConstraint() {
		// TODO Auto-generated method stub
		return null;
	}

	public String getDoFExplorationStrategy() {
		// TODO Auto-generated method stub
		return null;
	}

	public Properties getDoFExplorationStrategyProperties() {
		// TODO Auto-generated method stub
		return null;
	}

	public List<String> getSupportedConstraintNames() {
		// TODO Auto-generated method stub
		return null;
	}

	public List<String> getSupportedDoFConstraintNames() {
		// TODO Auto-generated method stub
		return null;
	}

	public List<String> getSupportedExplorationStrategyNames() {
		// TODO Auto-generated method stub
		return null;
	}

	public List<String> getSupportedTimeUnits() {
		// TODO Auto-generated method stub
		return null;
	}

	private EntityMapping loadObservationInstance(final ObservationRepository repository) {
		final EList<Observation> observations = repository.getObservations();
		final int numObservations = observations.size();

		if (!(observationAmountUnits < numObservations)) {
			final String message = "Requested instance before too large. Only " + numObservations + " steps available.";

			log.error(message);
			throw new IllegalArgumentException(message);
		}

		final int requestedIndex = (int) (numObservations - observationAmountUnits - 1);
		return observations.get(requestedIndex).getObservation();
	}

	public List<EntityMapping> requestDoFMetrics(final EntityMapping request) {
		// TODO Auto-generated method stub
		return null;
	}

	public EntityMapping requestMetrics(final EntityMapping request) {
		final EntityMapping response = EcoreUtil.copy(request);
		final String modelLocation = RepositoryHelper.qualifyModelLocation(request.getModelLocation());

		// open model
		final ObservationRepository repository = RepositoryHelper.loadModelInstance(modelLocation);

		EntityMapping dataSource = null;

		// compute mapping for slicing
		if (!withObserve) {
			final Observation latestObservation = repository.getLatestObservation();
			dataSource = latestObservation.getObservation();
		} else {
			if (!framedObservation) {
				if (observationByReference && observationUnit.equals(IMPACT_REFERENCE)) {
					// Currently no caching mechanism is implemented, therefore
					// slicing before calculating the impact would be
					// intelligent. But as the model instances are small and
					// caching intermediate results is a viable option for the
					// future, slicing afterwards makes sense and motivates the
					// implementation of efficient caching. ;)
					dataSource = createInstanceFromImpact(response, repository);
				} else if (!observationByReference && observationUnit.equals(BEFORE_UNIT)) {
					dataSource = loadObservationInstance(repository);
				} else {
					final String message = "Specified reference not supported";
					log.error(message);

					throw new IllegalArgumentException(message);
				}

			} else {
				final String message = "Specified observation not supported";
				log.error(message);

				throw new IllegalArgumentException(message);
			}
		}

		createResponseFromInstance(response, dataSource);

		return response;
	}

	public void setConstraint(final String constraint) {
		// TODO Auto-generated method stub

	}

	public void setDoFConstraint(final String doFConstraint) {
		// TODO Auto-generated method stub

	}

	public void setDoFExplorationStrategy(final String strategy, final Properties properties) {
		// TODO Auto-generated method stub

	}

	public void setObservationPeriod(final Date start, final Date end, final long sampleSize) {
		observationStartDate = start;
		observationEndDate = end;
		observationSampleSize = sampleSize;
		framedObservation = true;
		withObserve = true;
	}

	public void setObservationPeriod(final long amount, final String unit, final long sampleSize) {
		observationAmountUnits = amount;
		observationUnit = unit;
		observationSampleSize = sampleSize;
		framedObservation = false;
		withObserve = true;
	}

	public void setObservationReference(final String type, final String identifier, final long sampleSize) {
		observationUnit = type;
		observationIdentifier = identifier;
		observationSampleSize = sampleSize;
		withObserve = true;
		framedObservation = false;
		observationByReference = true;
	}

	public boolean supportsConstraintRequests() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean supportsDoFRequests() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean supportsMetricRequests() {
		return true;
	}

	public boolean supportsObservationPeriods() {
		return true;
	}

	public void reset() {
		observationStartDate = null;
		observationEndDate = null;
		observationSampleSize = 0;
		withObserve = false;
		framedObservation = false;
		observationByReference = false;
		observationUnit = null;
		observationIdentifier = null;
		observationAmountUnits = 0;
	}
}
