package edu.kit.ipd.descartes.ql.connector.pdr;

import java.io.File;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;

import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Probe;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;
import edu.kit.ipd.descartes.ql.models.repository.ObservationRepository;

public class RepositoryHelper {

	public static Map<String, Entity> buildEntityIndex(final EntityMapping mapping) {
		final List<Entity> entities = new LinkedList<Entity>();

		entities.addAll(mapping.getResources());
		entities.addAll(mapping.getServices());

		final Map<String, Entity> entityIndex = new HashMap<String, Entity>(entities.size());

		for (final Entity e : entities) {
			entityIndex.put(e.getIdentifier(), e);
		}

		return entityIndex;
	}

	public static Map<String, Probe> buildProbeIndex(final Entity latestEntity) {
		final EList<Probe> probes = latestEntity.getProbes();
		final Map<String, Probe> latestProbeIndex = new HashMap<String, Probe>(probes.size());

		for (final Probe p : probes) {
			latestProbeIndex.put(p.getMetricName(), p);
		}

		return latestProbeIndex;
	}

	public static ObservationRepository loadModelInstance(final String modelLocation) {
		final ResourceSet resourceSet = new ResourceSetImpl();
		final URI modelURI = URI.createFileURI(modelLocation);
		final org.eclipse.emf.ecore.resource.Resource model = resourceSet.getResource(modelURI, true);

		return (ObservationRepository) model.getContents().get(0);
	}

	public static String qualifyModelLocation(final String modelLocation) {
		final IWorkspace workspace = ResourcesPlugin.getWorkspace();
		final IWorkspaceRoot root = workspace.getRoot();
		final IPath location = root.getLocation();
		final IPath modelPath = location.append(modelLocation);
		final File modelFile = modelPath.toFile();

		if (!modelFile.isFile()) {
			throw new IllegalArgumentException(modelLocation + " is not available");
		}

		return modelPath.toString();
	}

}
