package edu.kit.ipd.descartes.ql.connector;

/**
 * This class is exposed by a connector to provide an entry point of creating
 * instances of query connectors for specific query classes
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public interface ConnectorProvider {
	/**
	 * Name of the connector properties file
	 */
	public static String PROPERTIES_FILE = "META-INF/connector.properties";

	/**
	 * The property key for the name of the performance meta-model family
	 * supported
	 */
	public static String FAMILY_KEY = "dql.connector.family";

	/**
	 * The property key for indicating support of the model structure query
	 * class
	 */
	public static String MODEL_STRUCTURE_SUPPORT_KEY = "dql.connector.modelstructure";

	/**
	 * The property key for indicating support of the performance metrics query
	 * class
	 */
	public static String PERFORMANCE_METRICS_SUPPORT_KEY = "dql.connector.performancemetrics";

	/**
	 * The property key for indicating support of the performance issues query
	 * class
	 */
	public static String PERFORMANCE_ISSUES_SUPPORT_KEY = "dql.connector.performanceissues";

	/**
	 * Instantiate a ModelStructureQueryConnector
	 * 
	 * @see ModelStructureQueryConnector
	 * @return An instance of {@link ModelStructureQueryConnector}
	 */
	public ModelStructureQueryConnector createModelStructureQueryConnectorInstance();

	/**
	 * Instantiate a PerformanceIssueQueryConnector
	 * 
	 * @see PerformanceIssueQueryConnector
	 * @return An instance of {@link PerformanceIssueQueryConnector}
	 */
	public PerformanceIssueQueryConnector createPerformanceIssueQueryConnectorInstance();

	/**
	 * Instantiate a PerformanceMetricsQueryConnector
	 * 
	 * @see PerformanceMetricsQueryConnector
	 * @return An instance of {@link PerformanceMetricsQueryConnector}
	 */
	public PerformanceMetricsQueryConnector createPerformanceMetricsQueryConnectorInstance();
}
