package edu.kit.ipd.descartes.ql.connector;

import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

/**
 * This interface defines the necessary methods to execute queries for model
 * structure-related information. The information retrieved can be used to form
 * queries of a higher order, e.g. to retrieve performance metrics.
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public interface ModelStructureQueryConnector extends QueryConnector {
	public EntityMapping requestDoF(EntityMapping request);

	/**
	 * Request the connector to resolve all entities from a performance model
	 * instance and apply an optional filter condition
	 * 
	 * @param request
	 *            A mapping model instance with optional filters
	 * @return The mapping model instance with all resolved entities attached
	 */
	public EntityMapping requestEntities(EntityMapping request);

	/**
	 * Request the connector to resolve all computable probes for the entities
	 * attached to this mapping model instance
	 * 
	 * @param request
	 *            A mapping model instance with performance model instance
	 *            entities
	 * @return The mapping model instance with computable probes attached to the
	 *         model entities
	 */
	public EntityMapping requestProbes(EntityMapping request);

	 //TODO
	public boolean supportsDoFRequests();

	/**
	 * Test whether entity requests are supported by the implementing class
	 * 
	 * @return True if entity requests are supported, false otherwise
	 */
	public boolean supportsEntityRequests();

	/**
	 * Test whether probe requests are supported by the implementing class
	 * 
	 * @return True if probe requests are supported, false otherwise
	 */
	public boolean supportsProbeRequests();
}
