package edu.kit.ipd.descartes.ql.connector;

import java.util.List;

import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

/**
 * This interface defines the necessary methods to execute queries to detect
 * performance issues. The information retrieved can be used to identify
 * optimization potential of entities in performance model instances.
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public interface PerformanceIssueQueryConnector extends StatefulQueryConnector {
	/**
	 * Get the list of supported exploration strategy names
	 * 
	 * @return A list of exploration strategy names
	 */
	public List<String> getSupportedExplorationStrategyNames();

	/**
	 * Request the connector to compute and detect all possible bottlenecks in a
	 * performance model instance
	 * 
	 * @param request
	 *            A mapping model consisting of model entities and probes to be
	 *            computed
	 * @return A mapping model instance with bottlenecks
	 */
	public EntityMapping requestBottleneckDetection(EntityMapping request);

	/**
	 * Set the name of the exploration strategy used for parameter space
	 * exploration in a query with degrees of freedom
	 * 
	 * @see #getSupportedExplorationStrategyNames()
	 * 
	 * @param explorationStrategy
	 *            The name of an exploration strategy
	 */
	public void setBottleneckDetectionExplorationStrategy(String explorationStrategy);

	/**
	 * Test whether bottleneck detection requests are supported by the
	 * implementing class
	 * 
	 * @return True if bottleneck detection requests are supported, false
	 *         otherwise
	 */
	public boolean supportsBottleneckDetectionRequests();

}
