package edu.kit.ipd.descartes.ql.core.engine.aggregates;

import org.apache.log4j.Logger;

import edu.kit.ipd.descartes.ql.connector.util.DQLLogger;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.AggregateCalculator;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.GeoMeanAggregateCalculator;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.MaxAggregateCalculator;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.MeanAggregateCalculator;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.MinAggregateCalculator;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.NAggregateCalculator;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.PercentileAggregateCalculator;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.StdDevAggregateCalculator;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.SumAggregateCalculator;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.SumOfSquaresAggregateCalculator;
import edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators.VarAggregateCalculator;
import edu.kit.ipd.descartes.ql.lang.descartesQL.StatisticalAggregateType;

/**
 * Creates an instance of a AggregateCalculator for a specific aggregate type
 * indicated by StatisticalAggregateType
 * 
 * @see AggregateCalculator
 * @see StatisticalAggregateType
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 * 
 */
public class AggregateCalculatorFactory {
	private static Logger log = DQLLogger.getLogger(AggregateCalculatorFactory.class.getName());

	/**
	 * Create an AggregateCalcultor based on the requested type
	 * 
	 * @param function
	 *            The aggregate type specified in the query language
	 *            infrastructure
	 * @return A new instance of an AggregateCalculator capable of the
	 *         StatisticalAggregateType
	 */
	public static AggregateCalculator createCalculator(final StatisticalAggregateType function) {
		final String message = "Requested aggregate function unknown";

		switch (function) {
		case GEOMEAN:
			return new GeoMeanAggregateCalculator();
		case MAX:
			return new MaxAggregateCalculator();
		case MEAN:
			return new MeanAggregateCalculator();
		case MIN:
			return new MinAggregateCalculator();
		case N:
			return new NAggregateCalculator();
		case PERCENTILE:
			return new PercentileAggregateCalculator();
		case STDDEV:
			return new StdDevAggregateCalculator();
		case SUM:
			return new SumAggregateCalculator();
		case SUMOFSQUARES:
			return new SumOfSquaresAggregateCalculator();
		case VAR:
			return new VarAggregateCalculator();
		default:
			log.error(message);
			throw new RuntimeException(message);
		}
	}
}
