package edu.kit.ipd.descartes.ql.core.engine.aggregates;

import java.util.Map;

import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

/**
 * An AggregateController is responsible for the registration of aggregates,
 * their pre- and post-processing.
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 * 
 */
public interface AggregateController {

	/**
	 * Tests whether aggregates are registered
	 * 
	 * @return If true, aggregates are registered and processing is required
	 */
	public boolean hasAggregates();

	/**
	 * After the connector returns an EntityMapping a result, the aggregate
	 * values can be calculated and injected Probes can be removed from the
	 * response
	 * 
	 * @param response
	 *            The result of the query execution returned by the
	 *            corresponding connector
	 */
	public void postProcess(EntityMapping response);

	/**
	 * If aggregates have been registered, they must be pre-processed in order
	 * to ensure all requirements are met to calculate the aggregates after the
	 * query has been executed
	 * 
	 * @param request
	 *            The request to be sent to the query connector
	 * @param aliasMap
	 *            A map of metrics with aliased entities in the query
	 */
	public void preProcess(EntityMapping request, Map<String, Entity> aliasMap);

	/**
	 * Register an AggregateProcessor for an aggregate found in the query
	 * 
	 * @param processor
	 *            The AggregateProcessor to be registered
	 */
	public void registerAggregate(AggregateProcessor processor);

}
