package edu.kit.ipd.descartes.ql.core.engine.aggregates;

import java.util.Map;
import java.util.Set;

import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Probe;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

/**
 * An AggregateProcessor is responsible for the calculation of a single
 * aggregate found in a query. The AggregateProcessor performs all necessary
 * steps during pre- and post-processing and instantiates an AggregateCalculator
 * usable to calculate the requested aggregate.
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 * 
 */
public interface AggregateProcessor {
	/**
	 * Post-processing of an aggregate consisting of the calculation of the
	 * aggregate value
	 * 
	 * @param response
	 *            The response to be returned as query execution result
	 * @param entityIndex
	 *            An index structure to look-up entities by their identifier
	 * @param probeIndex
	 *            An index structure to look-up Probes by their metricName by
	 *            entity
	 */
	public void postProcess(EntityMapping response, Map<String, Entity> entityIndex,
			Map<String, Map<String, Probe>> probeIndex);

	/**
	 * Pre-processing of aggregates consists of determining if all required
	 * Probes are set on all required Entities
	 * 
	 * @param request
	 *            The request to be sent to the query connector
	 * @param aliasMap
	 *            A map of aliases for entities
	 * @return An index structure of Probes that need to be added to the request
	 *         in order to calculate the requested aggregate
	 */
	public Map<String, Set<String>> preProcess(EntityMapping request, Map<String, Entity> aliasMap);
}
