package edu.kit.ipd.descartes.ql.core.engine.aggregates.calculators;

import java.math.BigDecimal;
import java.util.Properties;

import org.apache.log4j.Logger;

import edu.kit.ipd.descartes.ql.connector.util.DQLLogger;

/** 
 * Implementation of an AggregateController to compute the n-th percentile
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public class PercentileAggregateCalculator extends AbstractDescriptiveStatisticsAggregateCalculator implements
		AggregateWithProperties {
	public static final String PERCENTILE = "percentile";
	private final Logger log = DQLLogger.getLogger(this.getClass().getName());
	private Properties properties;

	private double getRequestedPercentile() {
		return Double.parseDouble(properties.getProperty(PERCENTILE));
	}

	public BigDecimal getResult() {
		if (properties == null) {
			log.error("");
			return BigDecimal.ZERO;
		}

		final double value = getStatistics().getPercentile(getRequestedPercentile());
		return new BigDecimal(value);
	}

	public boolean isValid() {
		if (properties == null) {
			return false;
		}

		final double value = getStatistics().getPercentile(getRequestedPercentile());
		return !Double.isNaN(value);
	}

	public void setProperties(final Properties properties) {
		this.properties = properties;
	}

}
