package edu.kit.ipd.descartes.ql.core.engine.interpreter;

import java.util.List;

import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

/**
 * Interface defining the interpretation of Xtext's incoming model and
 * transforms the model into instances of the Mapping Meta-Model, executes the
 * query and stores the result after query execution.
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public interface QueryInterpreter {
	/**
	 * Returns the first result or in case of a single result set, the only
	 * result.
	 * 
	 * @return The first result of the query execution
	 */
	public EntityMapping getResult();

	/**
	 * Returns all available query results.
	 * 
	 * @return All available results of the query execution
	 */
	public List<EntityMapping> getResults();

	/**
	 * Returns if the interpretation or execution of the query led to multiple
	 * results.
	 * 
	 * @return True, if multiple results are available, false otherwise
	 */
	public boolean hasMultipleResults();

	/**
	 * Starts the interpretation process of an Xtext processed model instance.
	 * 
	 * If the interpretation was successful, the results can be retrieved
	 * through the available getter methods.
	 * 
	 * @see #getResult()
	 * @see #getResults()
	 * 
	 * @return True, if the validation and interpretation was executed
	 *         successfully, otherwise false
	 */
	public boolean interpretQuery();
}
