package edu.kit.ipd.descartes.ql.core.engine.observations;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.log4j.Logger;

import edu.kit.ipd.descartes.ql.connector.util.DQLLogger;
import edu.kit.ipd.descartes.ql.lang.descartesQL.AbsoluteTimeClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.TimeClause;

public class AbsoluteTimeCalculator implements TimeCalculator {
	public final String DATE_DAY = "yyyy-MM-dd";
	public final String DATE_SEC = "yyyy-MM-dd'T'HH:mm:ss";
	public final String DATE_SEC_TZ = "yyyy-MM-dd'T'HH:mm:ssZ";
	public final String DATE_MSEC = "yyyy-MM-dd'T'HH:mm:ss.SSS";
	public final String DATE_MSEC_TZ = "yyyy-MM-dd'T'HH:mm:ss.SSSZ";

	public final String[] DATE_FORMATS = new String[] { DATE_MSEC_TZ, DATE_MSEC, DATE_SEC_TZ, DATE_SEC, DATE_DAY };

	private final Logger log = DQLLogger.getLogger(this.getClass().getName());
	private Date date = null;
	private boolean valid = false;

	public AbsoluteTimeCalculator() {
		super();
	}

	public AbsoluteTimeCalculator(final TimeClause timeSpecification) {
		parseDate(timeSpecification);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * edu.kit.ipd.descartes.ql.core.engine.observations.TimeCalculator#getDate
	 * ()
	 */
	public Date getDate() {
		return date;
	}

	public boolean isBackwards() {
		return false;
	}

	public boolean isValid() {
		return valid;
	}

	public boolean parseDate(final TimeClause timeClause) {
		final AbsoluteTimeClause absoluteTimeClause = (AbsoluteTimeClause) timeClause;
		final String timeSpecification = absoluteTimeClause.getTimeSpec();
		final SimpleDateFormat dateFormat = new SimpleDateFormat();
		Date parsedDate = null;

		// Use strict parsing!
		dateFormat.setLenient(false);

		for (final String element : DATE_FORMATS) {
			dateFormat.applyPattern(element);

			try {
				parsedDate = dateFormat.parse(timeSpecification);
			} catch (final ParseException e) {
				log.info(e.getMessage());
			}

			// Got a date, we're done
			if (parsedDate != null) {
				date = parsedDate;
				valid = true;
				return true;
			}
		}

		return false;
	}

	public void setBackwards(final boolean backwards) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.kit.ipd.descartes.ql.core.engine.observations.TimeCalculator#
	 * setReferenceDate(java.util.Date)
	 */
	public void setReferenceDate(final Date referenceDate) {
		// As this class is used to work on AbsoluteTimeClauses, we do not need
		// a reference date
	}

}
