package edu.kit.ipd.descartes.ql.core.engine.observations;

import java.util.Date;

import edu.kit.ipd.descartes.ql.connector.PerformanceMetricsQueryConnector;
import edu.kit.ipd.descartes.ql.lang.descartesQL.AbsoluteTimeClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ObserveRelativeClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ObserveRelativeDirectionType;
import edu.kit.ipd.descartes.ql.lang.descartesQL.TimeClause;

public class ObserveRelativeInterpreter implements ObservationInterpreter {
	private final ObserveRelativeClause observation;
	private boolean referenceTimeAsEnd = true;
	private final Date referenceDate = new Date();
	private Date otherTime = null;

	public ObserveRelativeInterpreter(final ObserveRelativeClause observation) {
		this.observation = observation;
		interpretDate();
	}

	public void configureConnector(final PerformanceMetricsQueryConnector queryConnector, final long sampleSize) {
		if (referenceTimeAsEnd) {
			queryConnector.setObservationPeriod(otherTime, referenceDate, sampleSize);
		} else {
			queryConnector.setObservationPeriod(referenceDate, otherTime, sampleSize);
		}

	}

	public void interpretDate() {
		final ObserveRelativeDirectionType direction = observation.getDirection();
		final TimeClause timeTarget = observation.getTimeTarget();

		if (direction.equals(ObserveRelativeDirectionType.SINCE)) {
			referenceTimeAsEnd = true;
		} else {
			referenceTimeAsEnd = false;
		}

		TimeCalculator calc = null;

		if (timeTarget instanceof AbsoluteTimeClause) {
			calc = new AbsoluteTimeCalculator(timeTarget);
		} else {
			calc = new RelativeTimeCalculator();
			calc.setBackwards(referenceTimeAsEnd);
			calc.parseDate(timeTarget);
		}

		if (calc.isValid()) {
			otherTime = calc.getDate();
		}
	}

}
