package edu.kit.ipd.descartes.ql.core.engine.observations;

import static edu.kit.ipd.descartes.ql.core.engine.observations.TimeModifierConverter.convert;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import edu.kit.ipd.descartes.ql.lang.descartesQL.RelativeTimeClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.RelativeTimeDurationClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.TimeClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.TimeModifierType;

public class RelativeTimeCalculator implements TimeCalculator {
	private final boolean valid = false;

	private boolean backwards = false;
	private long millis = 0;
	private Date referenceDate;

	public RelativeTimeCalculator() {
		super();
	}

	public Date getDate() {
		final Calendar calendar = new GregorianCalendar();

		if (referenceDate != null) {
			calendar.setTime(referenceDate);
		} else {
			// Ensure reentrant behavior
			referenceDate = calendar.getTime();
		}

		final long targetedTime = calendar.getTimeInMillis() + millis;
		calendar.setTimeInMillis(targetedTime);

		final Date targetedDate = calendar.getTime();
		return targetedDate;
	}

	public boolean isBackwards() {
		return backwards;
	}

	public boolean isValid() {
		return valid;
	}

	public boolean parseDate(final TimeClause timeClause) {
		final RelativeTimeClause relativeTimeClause = (RelativeTimeClause) timeClause;
		millis = 0;

		for (final RelativeTimeDurationClause r : relativeTimeClause.getDurationClause()) {
			final int duration = r.getDuration();
			final TimeModifierType timeModifier = r.getTimeModifier();

			millis += duration * convert(timeModifier);
		}

		if (backwards == true) {
			millis *= (-1l);
		}

		return true;
	}

	public void setBackwards(final boolean backwards) {
		// If the value switched the direction, invert it
		if (this.backwards ^ backwards) {
			millis *= (-1l);
		}

		this.backwards = backwards;
	}

	public void setReferenceDate(final Date referenceDate) {
		this.referenceDate = referenceDate;
	}
}
