package edu.kit.ipd.descartes.ql.core.engine.observations;

import static edu.kit.ipd.descartes.ql.lang.descartesQL.TimeModifierType.DAY;
import static edu.kit.ipd.descartes.ql.lang.descartesQL.TimeModifierType.HOUR;
import static edu.kit.ipd.descartes.ql.lang.descartesQL.TimeModifierType.MILLISECOND;
import static edu.kit.ipd.descartes.ql.lang.descartesQL.TimeModifierType.MINUTE;
import static edu.kit.ipd.descartes.ql.lang.descartesQL.TimeModifierType.MONTH;
import static edu.kit.ipd.descartes.ql.lang.descartesQL.TimeModifierType.SECOND;
import static edu.kit.ipd.descartes.ql.lang.descartesQL.TimeModifierType.WEEK;
import edu.kit.ipd.descartes.ql.lang.descartesQL.TimeModifierType;

/**
 * Calculates the factor for a given time modifier
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public class TimeModifierConverter {
	public static final long DAY_MS = 24l * convert(HOUR);
	public static final long HOUR_MS = 60l * convert(MINUTE);
	public static final long MINUTE_MS = 60l * convert(SECOND);

	/**
	 * Assume each month consists of four WEEK_MS
	 */
	public static final long MONTH_MS = 4l * convert(WEEK);

	public static final long SECOND_MS = 1000l * convert(MILLISECOND);
	public static final long WEEK_MS = 7l * convert(DAY);
	public static final long YEAR_MS = 12l * convert(MONTH);
	public static final long NULL_MS = 0l;
	public static final long ONE_MS = 1l;

	/**
	 * Conversion of a TimeModifierType into its representation as milliseconds.
	 * The conversion is based on the preceding (i.e. smaller) time unit with
	 * recursion until the smallest time unit supported (i.e. milliseconds) is
	 * reached. The recursion can be resolved by the compiler.
	 * 
	 * @param modifier
	 * @return
	 */
	public static long convert(final TimeModifierType modifier) {
		switch (modifier) {
		case DAY:
			return DAY_MS;
		case HOUR:
			return HOUR_MS;
		case MILLISECOND:
			return ONE_MS;
		case MINUTE:
			return MINUTE_MS;
		case MONTH:
			return MONTH_MS;
		case SECOND:
			return SECOND_MS;
		case WEEK:
			return WEEK_MS;
		case YEAR:
			return YEAR_MS;
		default:
			return NULL_MS;
		}
	}
}
