package edu.kit.ipd.descartes.ql.core.engine.query;

import java.util.List;

import org.eclipse.emf.ecore.resource.Resource;

import edu.kit.ipd.descartes.ql.lang.descartesQL.DescartesQL;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

/**
 * Interface to the query interpretation process
 * 
 * @author Fabian Gorsler <fabian.gorsler@student.kit.edu>
 */
public interface QueryExecutionEngine {
	/**
	 * Execute the query interpretation
	 * 
	 * @param query
	 *            The Ecore model instance, i.e. the query, to be used
	 */
	public List<EntityMapping> execute(DescartesQL query);

	/**
	 * Execute the query interpretation
	 * 
	 * @param model
	 *            The EMF Resource resource to be used as input
	 */
	public List<EntityMapping> execute(Resource model);

	/**
	 * Execute the query interpretation
	 * 
	 * @param query
	 *            The textual representation of a DQL query to be parsed
	 */
	public List<EntityMapping> execute(String query);

	/**
	 * Returns the success state of the query execution
	 * 
	 * This method should be called after query execution using
	 * {@link #execute()} and before {@link #getResults()}.
	 * 
	 * @see #execute()
	 * @see #getResults()
	 * 
	 * @return True, if the query execution was successful, false otherwise.
	 */
	public boolean isValidQuery();
}
