package edu.kit.ipd.descartes.ql.core.engine.query;

import static edu.kit.ipd.descartes.ql.core.engine.util.QueryParserHelper.stringToModel;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;

import edu.kit.ipd.descartes.ql.connector.util.DQLLogger;
import edu.kit.ipd.descartes.ql.core.engine.interpreter.ModelStructureQueryInterpreter;
import edu.kit.ipd.descartes.ql.core.engine.interpreter.PerformanceMetricsQueryInterpreter;
import edu.kit.ipd.descartes.ql.core.engine.interpreter.QueryInterpreter;
import edu.kit.ipd.descartes.ql.lang.descartesQL.DescartesQL;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ModelStructureQuery;
import edu.kit.ipd.descartes.ql.lang.descartesQL.PerformanceIssueQuery;
import edu.kit.ipd.descartes.ql.lang.descartesQL.PerformanceMetricsQuery;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

public class QueryExecutionEngineImpl implements QueryExecutionEngine {
	private final Logger log = DQLLogger.getLogger(this.getClass().getName());
	private boolean validQuery = false;
	private EObject query = null;

	private ArrayList<EntityMapping> createEmptyResultList() {
		return new ArrayList<EntityMapping>(0);
	}

	private List<EntityMapping> execute() {
		QueryInterpreter interpreter;

		validQuery = false;

		if (query == null) {
			log.error("Query is null, aborting.");
			return createEmptyResultList();
		}

		// Pick the right query interpreter for the incoming query instance
		if (query instanceof ModelStructureQuery) {
			interpreter = new ModelStructureQueryInterpreter((ModelStructureQuery) query);
		} else if (query instanceof PerformanceMetricsQuery) {
			interpreter = new PerformanceMetricsQueryInterpreter((PerformanceMetricsQuery) query);
		} else if (query instanceof PerformanceIssueQuery) {
			log.error("Performance Issues not yet supported, aborting.");
			return createEmptyResultList();
		} else {
			log.error("No suitable QueryInterpreter found, aborting.");
			return createEmptyResultList();
		}

		// Store state
		validQuery = interpreter.interpretQuery();
		// Pass back to GC
		query = null;

		return interpreter.getResults();
	}

	public List<EntityMapping> execute(final DescartesQL query) {
		setModel(query);
		return execute();
	}

	public List<EntityMapping> execute(final Resource model) {
		setModel(model);
		return execute();
	}

	public List<EntityMapping> execute(final String query) {
		setModel(query);
		return execute();
	}

	public boolean isValidQuery() {
		return validQuery;
	}

	private void setModel(final DescartesQL query) {
		this.query = query.getQuery();
	}

	private void setModel(final Resource model) {
		EList<EObject> contents = null;

		if (model == null) {
			log.error("The model instance is null, aborting.");
			return;
		}

		contents = model.getContents();

		if (contents.size() != 1) {
			log.error("Invalid model structure, more than one root element");
			return;
		}

		query = ((DescartesQL) contents.get(0)).getQuery();
	}

	private void setModel(final String query) {
		final DescartesQL queryModel = stringToModel(query);
		this.query = queryModel.getQuery();
	}
}
