package edu.kit.ipd.descartes.ql.core.engine.util;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.util.EList;

import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Probe;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Resource;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Service;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

public class EntityMappingHelper {

	/*
	 * Builds an index of entities using the identifier as key
	 */
	public static Map<String, Entity> buildEntityIndex(final EntityMapping response) {
		final List<Resource> resources = response.getResources();
		final List<Service> services = response.getServices();
		final int size = resources.size() + services.size();
		final HashMap<String, Entity> index = new HashMap<String, Entity>(size);

		// Code doubling, but still more reasonable than copying both lists into
		// a temporary one first and then iterating over the list in a single
		// loop
		for (final Entity e : resources) {
			index.put(e.getIdentifier(), e);
		}

		for (final Entity e : services) {
			index.put(e.getIdentifier(), e);
		}

		return index;
	}

	public static Map<String, Map<String, Probe>> buildProbeIndex(final EntityMapping mapping) {
		final EList<Resource> resources = mapping.getResources();
		final EList<Service> services = mapping.getServices();
		final int numEntities = resources.size() + services.size();
		final Map<String, Map<String, Probe>> map = new HashMap<String, Map<String, Probe>>(numEntities);

		for (final Resource r : resources) {
			map.put(r.getIdentifier(), EntityMappingHelper.indexProbes(r));
		}

		for (final Service s : services) {
			map.put(s.getIdentifier(), EntityMappingHelper.indexProbes(s));
		}

		return map;
	}

	public static Map<String, Probe> indexProbes(final Entity entity) {
		final EList<Probe> probes = entity.getProbes();
		final Map<String, Probe> probeIndex = new HashMap<String, Probe>(probes.size());

		for (final Probe p : probes) {
			final String metricName = p.getMetricName();
			probeIndex.put(metricName, p);
		}

		return probeIndex;
	}

}
