package edu.kit.ipd.descartes.ql.core.engine.util;

import static edu.kit.ipd.descartes.ql.connector.PerformanceMetricsQueryConnector.DOF_VARIATION_END_KEY;
import static edu.kit.ipd.descartes.ql.connector.PerformanceMetricsQueryConnector.DOF_VARIATION_FUNCTION_KEY;
import static edu.kit.ipd.descartes.ql.connector.PerformanceMetricsQueryConnector.DOF_VARIATION_INTERVAL_FUNCTION_VALUE;
import static edu.kit.ipd.descartes.ql.connector.PerformanceMetricsQueryConnector.DOF_VARIATION_START_KEY;
import static edu.kit.ipd.descartes.ql.connector.PerformanceMetricsQueryConnector.DOF_VARIATION_STEP_KEY;
import static edu.kit.ipd.descartes.ql.connector.PerformanceMetricsQueryConnector.DOF_VARIATION_VALUE_FUNCTION_VALUE;
import static edu.kit.ipd.descartes.ql.connector.PerformanceMetricsQueryConnector.DOF_VARIATION_VALUE_KEY;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.resource.XtextResourceSet;

import com.google.inject.Injector;

import edu.kit.ipd.descartes.ql.core.engine.Activator;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ConfigurationPropertiesClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ConfigurationPropertyClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.DescartesQL;
import edu.kit.ipd.descartes.ql.lang.descartesQL.IntervalVariationClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ValueVariationClause;

public class QueryParserHelper {

	public static Properties intervalToProperties(final IntervalVariationClause interval) {
		final Properties properties = new Properties();

		properties.put(DOF_VARIATION_FUNCTION_KEY, DOF_VARIATION_INTERVAL_FUNCTION_VALUE);
		properties.put(DOF_VARIATION_START_KEY, interval.getStart());
		properties.put(DOF_VARIATION_END_KEY, interval.getEnd());
		properties.put(DOF_VARIATION_STEP_KEY, interval.getStep());

		return properties;
	}

	/**
	 * Parse a String into a query http://wiki.eclipse.org/Xtext/FAQ
	 * 
	 * @param query
	 *            The String representing a query
	 * @return The DescartesQL model representing the query
	 */
	public static DescartesQL stringToModel(final String query) {
		// To access Xtext run-time already deployed
		final Injector injector = Activator.getDqlInjector();

		// EMF magic further obscured by dependency injection of Xtext
		final XtextResourceSet resourceSet = injector.getInstance(XtextResourceSet.class);
		resourceSet.addLoadOption(XtextResource.OPTION_RESOLVE_ALL, Boolean.TRUE);
		final Resource resource = resourceSet.createResource(URI.createURI("dummy:/url.dql"));
		final InputStream in = new ByteArrayInputStream(query.getBytes());

		try {
			resource.load(in, resourceSet.getLoadOptions());
		} catch (final IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		final DescartesQL model = (DescartesQL) resource.getContents().get(0);

		return model;
	}

	public static Properties toFlatConfigurationPropertiesClause(final ConfigurationPropertiesClause clause) {
		final Properties properties = new Properties();
		StringBuilder sbKey = null;
		String key = null;

		// TODO: Omit call to this method in case of null values
		if (clause == null) {
			return properties;
		}

		for (final ConfigurationPropertyClause property : clause.getProperties()) {
			sbKey = new StringBuilder();

			for (final String part : property.getKey()) {
				sbKey.append(part).append(".");
			}

			key = sbKey.substring(0, sbKey.length() - 1);
			properties.put(key, property.getValue());
		}

		return properties;
	}

	public static Properties valueToProperties(final ValueVariationClause value) {
		final Properties properties = new Properties();

		properties.put(DOF_VARIATION_FUNCTION_KEY, DOF_VARIATION_VALUE_FUNCTION_VALUE);
		properties.put(DOF_VARIATION_VALUE_KEY, value.getValue().toString());

		return properties;
	}
}
