package edu.kit.ipd.descartes.ql.core.platform.eclipse;

import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.AGGREGATE_ICON;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.AGGREGATE_ICON_KEY;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.DOF_ICON;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.DOF_ICON_KEY;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.RESOURCE_ICON;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.RESOURCE_ICON_KEY;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.SERVICE_ICON;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.SERVICE_ICON_KEY;

import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;

import com.google.inject.Guice;
import com.google.inject.Injector;

import edu.kit.ipd.descartes.ql.lang.DescartesQLRuntimeModule;

public class Activator extends AbstractUIPlugin implements BundleActivator {
	private static BundleContext context;

	private static Activator plugin;

	private static Injector dqlInjector;

	public static BundleContext getContext() {
		return context;
	}

	public static Injector getDqlInjector() {
		return dqlInjector;
	}

	public static Activator getPlugin() {
		return plugin;
	}

	public static void setDqlInjector(final Injector dqlInjector) {
		Activator.dqlInjector = dqlInjector;
	}

	public static void setPlugin(final Activator plugin) {
		Activator.plugin = plugin;
	}

	public Activator() {
		super();
		setPlugin(this);
	}

	@Override
	protected void initializeImageRegistry(final ImageRegistry reg) {
		super.initializeImageRegistry(reg);

		final ClassLoader classLoader = this.getClass().getClassLoader();
		final Display display = Display.getDefault();
		final ImageData serviceIcon = new ImageData(classLoader.getResourceAsStream(SERVICE_ICON));
		final ImageData aggregateIcon = new ImageData(classLoader.getResourceAsStream(AGGREGATE_ICON));
		final ImageData resourceIcon = new ImageData(classLoader.getResourceAsStream(RESOURCE_ICON));
		final ImageData doFIcon = new ImageData(classLoader.getResourceAsStream(DOF_ICON));

		reg.put(SERVICE_ICON_KEY, new Image(display, serviceIcon));
		reg.put(RESOURCE_ICON_KEY, new Image(display, resourceIcon));
		reg.put(AGGREGATE_ICON_KEY, new Image(display, aggregateIcon));
		reg.put(DOF_ICON_KEY, new Image(display, doFIcon));
	}

	@Override
	public void start(final BundleContext context) throws Exception {
		Activator.context = context;
		setDqlInjector(Guice.createInjector(new DescartesQLRuntimeModule()));
	}

	@Override
	public void stop(final BundleContext context) throws Exception {
		setDqlInjector(null);
		Activator.context = null;
	}

}
