package edu.kit.ipd.descartes.ql.core.platform.eclipse.launch;

import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.google.inject.Injector;

import edu.kit.ipd.descartes.ql.core.engine.query.QueryExecutionEngine;
import edu.kit.ipd.descartes.ql.core.engine.query.QueryExecutionEngineImpl;
import edu.kit.ipd.descartes.ql.core.platform.eclipse.Activator;
import edu.kit.ipd.descartes.ql.core.platform.eclipse.views.QueryResultTableViewer;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

class QueryExecutionEngineJob extends Job {
	public static final String DQL_JOB_NAME = "DQL Query Execution Engine";

	public static Resource resourceToModel(final IResource workspaceResource) {
		// To access Xtext run-time already deployed
		final Injector injector = Activator.getDqlInjector();

		// EMF magic further obscured by dependency injection of Xtext
		final ResourceSet resourceSet = injector.getInstance(ResourceSet.class);
		final String plainAbsoluteUri = workspaceResource.getLocationURI().toString();
		final URI pathUri = URI.createURI(plainAbsoluteUri);
		final Resource resource = resourceSet.getResource(pathUri, true);

		// Finally a model instance
		return resource;
	}

	private final QueryExecutionEngine engine = new QueryExecutionEngineImpl();
	private List<EntityMapping> results = null;

	private IResource resource = null;

	public QueryExecutionEngineJob(final IResource resource) {
		super(DQL_JOB_NAME);
		super.setPriority(Job.LONG);
		super.setUser(true);

		this.resource = resource;
	}

	private void openQueryResultTableViewer(final IWorkbenchPage activePage, final String sourceFile,
			final List<EntityMapping> results) throws PartInitException {
		int resultCounter = 0;

		for (final EntityMapping result : results) {
			final String queryTitle = sourceFile + "_" + ++resultCounter;
			final IViewPart view = activePage.showView(QueryResultTableViewer.ID, queryTitle,
					IWorkbenchPage.VIEW_CREATE);

			final QueryResultTableViewer qrtv = (QueryResultTableViewer) view;
			qrtv.setQueryTitle(queryTitle);
			qrtv.setQueryResult(result);
		}
	}

	@Override
	protected IStatus run(final IProgressMonitor monitor) {
		// Executed in dedicated thread in background
		final Resource emfResource = resourceToModel(resource);
		results = engine.execute(emfResource);

		// Execute in display thread (SWT-specific)
		Display.getDefault().asyncExec(new Runnable() {
			@Override
			public void run() {
				showQueryResult();
			}
		});

		return Status.OK_STATUS;
	}

	private void showQueryResult() {
		if (engine.isValidQuery()) {
			final IWorkbench workbench = PlatformUI.getWorkbench();
			final IWorkbenchWindow activeWorkbenchWindow = workbench.getActiveWorkbenchWindow();
			final IWorkbenchPage activePage = activeWorkbenchWindow.getActivePage();
			final String sourceFile = resource.getProjectRelativePath().lastSegment();

			try {
				openQueryResultTableViewer(activePage, sourceFile, results);
			} catch (final PartInitException e) {
				e.printStackTrace();
			}
		}
	}
}
