package edu.kit.ipd.descartes.ql.core.platform.eclipse.views;

import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.ACCURACY_POS;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.ALIAS_POS;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.ALIGNMENT;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.HEADINGS;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.IDENTIFIER_POS;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.METRIC_POS;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.TYPE_POS;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.VALID_POS;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.VALUE_POS;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.WEIGHTS;
import static edu.kit.ipd.descartes.ql.core.platform.eclipse.views.TableRowBean.WIDTHS;

import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.part.ViewPart;

import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;

public class QueryResultTableViewer extends ViewPart {
	/**
	 * The ID of the view as specified by the extension.
	 */
	public static final String ID = "edu.kit.ipd.descartes.ql.core.platform.eclipse.views.QueryResultTableViewer";

	private TableViewer viewer;

	private EntityMapping queryResult;

	private Composite composite;

	private TableColumnLayout layout;

	/**
	 * The constructor.
	 */
	public QueryResultTableViewer() {
	}

	private void createColumn(final TableViewer viewer, final TableColumnLayout layout, final int position) {
		final TableViewerColumn tableViewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn tableColumn = tableViewerColumn.getColumn();
		final String heading = HEADINGS[position];
		final int alignment = ALIGNMENT[position];
		final int weight = WEIGHTS[position];
		final int minimumWidth = WIDTHS[position];
		final ColumnWeightData columnWeight = new ColumnWeightData(weight, minimumWidth, true);

		layout.setColumnData(tableColumn, columnWeight);

		tableColumn.setText(heading);
		tableColumn.setToolTipText(heading);
		tableColumn.setResizable(true);
		tableColumn.setAlignment(alignment);
	}

	/**
	 * This is a callback that will allow us to create the viewer and initialize
	 * it.
	 */
	@Override
	public void createPartControl(final Composite parent) {
		composite = new Composite(parent, SWT.NONE);
		final int style = SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL;
		viewer = new TableViewer(composite, style);
		layout = new TableColumnLayout();

		composite.setLayout(layout);

		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		createColumn(viewer, layout, TYPE_POS);
		createColumn(viewer, layout, ALIAS_POS);
		createColumn(viewer, layout, IDENTIFIER_POS);
		createColumn(viewer, layout, METRIC_POS);
		createColumn(viewer, layout, VALID_POS);
		createColumn(viewer, layout, VALUE_POS);
		createColumn(viewer, layout, ACCURACY_POS);

		viewer.setContentProvider(new ViewContentProvider(this));
		viewer.setLabelProvider(new ViewLabelProvider());
		viewer.setComparator(new TableRowBeanComparator());

		if (queryResult != null) {
			viewer.setInput(queryResult);
		} else {
			viewer.setInput(getViewSite());
		}

		// Create the help context id for the viewer's control
		hookContextMenu();
	}

	private void hookContextMenu() {
		final MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		final Menu menu = menuMgr.createContextMenu(viewer.getControl());
		viewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, viewer);
	}

	public void refresh() {
		viewer.refresh();
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	@Override
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	public void setQueryResult(final EntityMapping queryResult) {
		this.queryResult = queryResult;
		viewer.setInput(queryResult);
	}

	public void setQueryTitle(final String title) {
		super.setPartName(title);
	}
}
