package edu.kit.ipd.descartes.ql.core.platform.eclipse.views;

import java.math.BigDecimal;
import java.util.ArrayList;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.Viewer;

import edu.kit.ipd.descartes.ql.models.mapping.domain.Aggregate;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DecimalResult;
import edu.kit.ipd.descartes.ql.models.mapping.domain.DoF;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Entity;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Probe;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Resource;
import edu.kit.ipd.descartes.ql.models.mapping.domain.Service;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.EntityMapping;
import edu.kit.ipd.descartes.ql.models.mapping.mapping.impl.EntityMappingImpl;

class ViewContentProvider implements IStructuredContentProvider {
	/**
	 * 
	 */
	private final QueryResultTableViewer queryResultTableViewer;

	/**
	 * @param queryResultTableViewer
	 */
	ViewContentProvider(final QueryResultTableViewer queryResultTableViewer) {
		this.queryResultTableViewer = queryResultTableViewer;
	}

	@Override
	public void dispose() {
	}

	@Override
	public Object[] getElements(final Object parent) {
		ArrayList<TableRowBean> rows;

		if (parent instanceof EntityMappingImpl) {
			rows = mappingToRows((EntityMapping) parent);
		} else {
			rows = new ArrayList<TableRowBean>();
		}

		return rows.toArray();
	}

	@Override
	public void inputChanged(final Viewer v, final Object oldInput, final Object newInput) {
		queryResultTableViewer.refresh();
	}

	private ArrayList<TableRowBean> mappingToRows(final EntityMapping mapping) {
		final ArrayList<TableRowBean> rows = new ArrayList<TableRowBean>();
		final ArrayList<Entity> entities = new ArrayList<Entity>();

		entities.addAll(mapping.getResources());
		entities.addAll(mapping.getServices());
		entities.addAll(mapping.getAggregates());
		entities.addAll(mapping.getDoF());

		for (final Entity entity : entities) {
			boolean hasProbes = false;
			final TableRowBean row = new TableRowBean();
			TableRowBean rowWithProbe;
			processModelEntity(row, entity);

			rowWithProbe = new TableRowBean(row.getType(), row.getAlias(), row.getIdentifier());

			for (final Probe probe : entity.getProbes()) {
				processProbe(rowWithProbe, probe);

				rows.add(rowWithProbe);
				rowWithProbe = new TableRowBean(rowWithProbe.getType(), rowWithProbe.getAlias(),
						rowWithProbe.getIdentifier());

				hasProbes = true;
			}

			if (!hasProbes) {
				rows.add(row);
			}
		}

		return rows;
	}

	private void processModelEntity(final TableRowBean row, final Entity entity) {
		if (entity instanceof Resource) {
			row.setType(TableRowBean.RESOURCE);
		} else if (entity instanceof Service) {
			row.setType(TableRowBean.SERVICE);
		} else if (entity instanceof DoF) {
			row.setType(TableRowBean.DOF);
		} else if (entity instanceof Aggregate) {
			row.setType(TableRowBean.AGGREGATE);
		} else {
			row.setType(TableRowBean.INVALID_TYPE);
		}

		row.setAlias(entity.getAlias());
		row.setIdentifier(entity.getIdentifier());
	}

	private void processProbe(final TableRowBean row, final Probe probe) {
		row.setMetricName(probe.getMetricName());

		if (probe instanceof DecimalResult) {
			final DecimalResult result = (DecimalResult) probe;

			row.setValid(result.getValid().toString());

			final BigDecimal scaledResult = scaleBigDecimal(result.getValue());
			final BigDecimal scaledAccuracy = scaleBigDecimal(result.getAccuracy());

			row.setValue(scaledResult.toString());
			if (scaledAccuracy != null) 
			    row.setAccuracy(scaledAccuracy.toString());
			else
			    row.setAccuracy("N/A");
		}
	}

	private BigDecimal scaleBigDecimal(final BigDecimal value) {

	    if (value!=null)
	        return value.setScale(TableRowBean.DECIMAL_SCALE, TableRowBean.DECIMAL_ROUNDING);
	    else
	        return null;
	}

}
