package edu.kit.ipd.descartes.ql.lang.examples.tests;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;

import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.junit.Test;

import edu.kit.ipd.descartes.ql.lang.descartesQL.AggregateMetricClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.AliasClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ConfigurationPropertiesClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ConfigurationPropertyClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ConstraintClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.DescartesQL;
import edu.kit.ipd.descartes.ql.lang.descartesQL.DoFClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.DoFReference;
import edu.kit.ipd.descartes.ql.lang.descartesQL.EntityReference;
import edu.kit.ipd.descartes.ql.lang.descartesQL.EntityReferenceClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.EntityType;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ExplorationStrategyClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ForClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.MetricClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.MetricClauseType;
import edu.kit.ipd.descartes.ql.lang.descartesQL.MetricReference;
import edu.kit.ipd.descartes.ql.lang.descartesQL.MetricReferenceClauses;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ObserveClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.RelativeTimeDurationClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.SelectQuery;
import edu.kit.ipd.descartes.ql.lang.descartesQL.TimeModifierType;
import edu.kit.ipd.descartes.ql.lang.descartesQL.VaryingClause;

public abstract class AbstractSelectQueryTest extends AbstractQueryTest implements SelectQueryTestIface {

	@Override
	public boolean expectConstraint() {
		return true;
	}

	@Override
	public boolean expectDoF() {
		return true;
	}

	public boolean expectDoFConstraint() {
		return true;
	}

	public boolean expectDoFGuidedBy() {
		return true;
	}

	public boolean expectDoFGuidedByParameters() {
		return true;
	}

	@Override
	public boolean expectObserve() {
		return true;
	}

	public boolean exploreDoFVarying() {
		return true;
	}

	public boolean exploreDoFWith() {
		return true;
	}

	public String getConstraintValue() {
		return null;
	}

	public DoFClause getDoFClause() {
		final SelectQuery select = getSelectQuery(getQuery());
		assertNotNull(select);

		final DoFClause dofClause = select.getDofClause();

		return dofClause;
	}

	public String getDoFConstraintName() {
		fail("Override this method");
		return null;
	}

	public String getDoFExplorationStrategyName() {
		fail("Override this method");
		return null;
	}

	public SelectQuery getSelectQuery(final DescartesQL query) {
		final EObject queryRoot = query.getQuery();

		assertNotNull("Casting to SelectQuery failed", queryRoot);

		return (SelectQuery) (queryRoot);
	}

	private String mergeKey(final EList<String> keyFragments) {
		final StringBuilder b = new StringBuilder();

		for (final String keyFragment : keyFragments) {
			b.append(keyFragment).append(".");
		}

		return b.substring(0, b.length() - 1);
	}

	public void testAggregate(final int i, final String string, final AggregateMetricClause aggregateMetric) {
		// Should be overridden if needed
		fail();
	}

	@Override
	@Test
	public void testConstraintClause() {
		final SelectQuery select = getSelectQuery(getQuery());
		assertNotNull(select);

		final ConstraintClause constraint = select.getConstraintClause();

		if (expectConstraint()) {
			assertNotNull("Please supply a valid constraint name in getConstraintValue()", getConstraintValue());

			final String type = constraint.getType();
			assertNotNull(type);
			assertEquals(getConstraintValue(), type);
		} else {
			assertNull("If no constraint is set, do not supply getConstraintValue()", getConstraintValue());
			assertNull(constraint);
		}
	}

	public void testDefaultForClause() {
		testForClause(new EntityType[] { EntityType.RESOURCE, EntityType.SERVICE }, new String[] { "thisIsAnId1",
				"thisIsAnId2" }, new String[] { "res1", "svc1" });
	}

	public void testDefaultMetricReferenceClause() {
		testMetricReferenceClause(new String[] { null, null, null, null }, new String[] { "res1", "res1", "svc1",
				"svc1" }, new String[] { "metric1", "metric2", "metric3", "metric4" });
	}

	@Override
	@Test
	public void testDoFClause() {
		final DoFClause dofClause = getDoFClause();

		if (expectDoF()) {
			assertNotNull(dofClause);

			if (exploreDoFWith()) {
				// Needs to be implemented
				fail();
			}

		} else {
			assertNull(dofClause);
		}
	}

	@Test
	public void testDoFConstraint() {
		if (!(expectDoF() && expectDoFConstraint())) {
			return;
		}

		final DoFClause dofClause = getDoFClause();
		final ConstraintClause constraintClause = dofClause.getConstraintClause();
		testDoFConstraintClause(constraintClause, getDoFConstraintName());
	}

	public void testDoFConstraintClause(final ConstraintClause constraintClause, final String expected) {
		assertNotNull(constraintClause);
		assertNotNull(expected);

		final String type = constraintClause.getType();
		assertNotNull(type);
		assertEquals(expected, type);
	}

	public void testDoFExplorationStrategyClause(final ExplorationStrategyClause explorationStrategy,
			final String expected) {
		assertNotNull(explorationStrategy);
		assertNotNull(expected);

		final String name = explorationStrategy.getName();
		assertNotNull(name);
		assertEquals(expected, name);
	}

	@Test
	public void testDoFGuidedBy() {
		if (!(expectDoF() && expectDoFGuidedBy())) {
			return;
		}

		final DoFClause dofClause = getDoFClause();
		final ExplorationStrategyClause explorationStrategy = dofClause.getExplorationStrategy();
		assertNotNull(explorationStrategy);

		testDoFExplorationStrategyClause(explorationStrategy, getDoFExplorationStrategyName());

		if (expectDoFGuidedByParameters()) {
			final ConfigurationPropertiesClause parameterClause = explorationStrategy.getParameterClause();
			testDoFGuidedByParameterClause(parameterClause);
		}
	}

	public void testDoFGuidedByParameterClause(final ConfigurationPropertiesClause parameterClause) {
		fail();
	}

	@Test
	public void testDoFVarying() {
		if (!(expectDoF() && exploreDoFVarying())) {
			return;
		}

		final DoFClause dofClause = getDoFClause();
		final VaryingClause varyingClause = dofClause.getVaryingClause();
		testDoFVaryingClause(varyingClause);
	}

	public void testDoFVaryingClause(final VaryingClause varyingClause) {
		fail();
	}

	public void testDoFVaryingClause(final VaryingClause varyingClause, final String[] identifiers,
			final String[] aliases) {
		assertNotNull(varyingClause);

		final EList<DoFReference> dofReferences = varyingClause.getDofReferences();
		assertNotNull(dofReferences);

		assertEquals(identifiers.length, aliases.length);
		assertEquals(identifiers.length, dofReferences.size());

		int i = 0;

		for (final DoFReference ref : dofReferences) {
			assertEquals(identifiers[i], ref.getDoFIdentifier());
			final AliasClause doFAliasClause = ref.getDoFAliasClause();

			if (aliases[i] == null) {
				assertNull(doFAliasClause);
			} else {
				assertEquals(aliases[i], doFAliasClause.getAlias());
			}

			i++;
		}
	}

	@Override
	@Test
	public void testForClause() {
		testDefaultForClause();
	}

	public void testForClause(final EntityType[] types, final String[] identifiers, final String[] aliases) {
		final SelectQuery select = getSelectQuery(getQuery());
		assertNotNull(select);

		final String inputLengthMsg = "Check input values, array length differs";
		assertEquals(inputLengthMsg, types.length, identifiers.length);
		assertEquals(inputLengthMsg, types.length, aliases.length);

		final ForClause forC = select.getForClause();
		assertNotNull(forC);

		final EntityReferenceClause refs = forC.getEntityReferenceClause();
		assertNotNull(refs);

		final EList<EntityReference> entityRefs = refs.getEntityReferences();
		assertNotNull(entityRefs);
		assertEquals(types.length, entityRefs.size());

		int i = 0;

		for (final EntityReference ref : entityRefs) {
			assertEquals("Type differs", types[i], ref.getType());
			assertEquals("Identier differs", identifiers[i], ref.getIdentifier());
			assertEquals("Alias differs", aliases[i], ref.getAlias().getAlias());

			i++;
		}
	}

	@Override
	public void testMetricReference(final MetricReference ref, final String idOrAlias, final String metric) {
		assertNotNull(ref);
		assertEquals(idOrAlias, ref.getEntityReference());
		assertEquals(metric, ref.getMetricReference());
	}

	@Override
	@Test
	public void testMetricReferenceClause() {
		testDefaultMetricReferenceClause();
	}

	public void testMetricReferenceClause(final List<MetricClauseType> refs, final String[] aggregate,
			final String[] idOrAlias, final String[] metric) {
		int i = 0;

		for (final MetricClauseType type : refs) {
			if (aggregate[i] == null) {
				final MetricClause clause = (MetricClause) type;
				assertNotNull(clause);

				final MetricReference ref = clause.getMetricReference();
				assertNotNull(ref);

				testMetricReference(ref, idOrAlias[i], metric[i]);
			} else {
				assertNotNull(type);

				if (!(type instanceof AggregateMetricClause)) {
					fail("Wrong test parameters");
				}

				testAggregate(i, aggregate[i], (AggregateMetricClause) type);
			}

			i++;
		}
	}

	public void testMetricReferenceClause(final String[] aggregate, final String[] idOrAlias, final String[] metric) {
		final SelectQuery select = getSelectQuery(getQuery());
		assertNotNull(select);

		assertEquals(aggregate.length, idOrAlias.length);
		assertEquals(aggregate.length, metric.length);

		final MetricReferenceClauses refs = select.getMetricReferenceClauses();
		assertNotNull(refs);

		final EList<MetricClauseType> metrics = refs.getMetricClauseTypes();
		assertNotNull(metrics);
		assertEquals("More test metrics than query metrics", aggregate.length, metrics.size());

		testMetricReferenceClause(metrics, aggregate, idOrAlias, metric);
	}

	@Override
	@Test
	public void testObserveClause() {
		final SelectQuery select = getSelectQuery(getQuery());
		assertNotNull(select);

		final ObserveClause observeClause = select.getObserveClause();

		if (expectObserve()) {
			assertNotNull(observeClause);
			testObserveClauseImpl(observeClause);
		} else {
			assertNull(observeClause);
		}

	}

	public void testObserveClauseImpl(final ObserveClause observeClause) {
		fail();
	}

	public void testParameterClause(final ConfigurationPropertiesClause parameterClause) {
		fail("Shall not be used, but the name of the properties clause shall be used."
				+ "See testDoFGuidedByParameterClause.");
	}

	public void testParameterClause(final ConfigurationPropertiesClause parameterClause, final String[] keys,
			final String[] values) {
		final EList<ConfigurationPropertyClause> properties = parameterClause.getProperties();
		assertNotNull(properties);

		assertNotNull(keys);
		assertNotNull(values);

		assertEquals(keys.length, values.length);
		assertEquals(keys.length, properties.size());

		int i = 0;

		for (final ConfigurationPropertyClause prop : properties) {
			final EList<String> keyFragments = prop.getKey();
			assertNotNull(keyFragments);

			final String key = mergeKey(keyFragments);
			assertEquals(keys[i], key);

			final String value = prop.getValue();
			assertEquals(values[i], value);

			i++;
		}
	}

	public void testRelativeTimeDuration(final EList<RelativeTimeDurationClause> durationClause, final int[] durations,
			final TimeModifierType[] modifiers) {
		assertNotNull(durationClause);
		assertEquals(durations.length, modifiers.length);
		assertEquals(durations.length, durationClause.size());

		int i = 0;

		for (final RelativeTimeDurationClause rel : durationClause) {
			assertEquals(durations[i], rel.getDuration());
			assertEquals(modifiers[i], rel.getTimeModifier());

			i++;
		}
	}

	@Override
	@Test
	public void testUsingClause() {
		testUsingClause(getSelectQuery(getQuery()).getUsingClause());
	}
}
