package edu.kit.ipd.descartes.ql.lang.examples.tests.list;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;

import org.eclipse.xtext.junit4.InjectWith;
import org.eclipse.xtext.junit4.XtextRunner;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

import edu.kit.ipd.descartes.ql.lang.DescartesQLInjectorProvider;
import edu.kit.ipd.descartes.ql.lang.descartesQL.EntityReference;
import edu.kit.ipd.descartes.ql.lang.descartesQL.EntityReferenceClause;
import edu.kit.ipd.descartes.ql.lang.descartesQL.EntityType;
import edu.kit.ipd.descartes.ql.lang.descartesQL.ListQuery;
import edu.kit.ipd.descartes.ql.lang.examples.tests.AbstractListQueryTest;

@RunWith(XtextRunner.class)
@InjectWith(DescartesQLInjectorProvider.class)
public class ListMetricsQuery extends AbstractListQueryTest {

	public static final String MODE = "METRICS";

	@Override
	@Before
	public void initializeTest() throws Exception {
		super.initialize(createFileName(LIST, this.getClass().getSimpleName()));
	}

	@Test
	@Override
	public void testFilterClause() {
		testNullFilterClause();
	}

	@Override
	@Test
	public void testListQueryMode() {
		final ListQuery list = getListQuery(getQuery());

		final edu.kit.ipd.descartes.ql.lang.descartesQL.ListMetricsQuery listMetrics = (edu.kit.ipd.descartes.ql.lang.descartesQL.ListMetricsQuery) list
				.getQueryType();

		assertEquals("Mode not set correctly", MODE, listMetrics.getMode());
	}

	@Test
	public void testMetricReferences() {
		final ListQuery list = getListQuery(getQuery());
		final edu.kit.ipd.descartes.ql.lang.descartesQL.ListMetricsQuery listMetrics = (edu.kit.ipd.descartes.ql.lang.descartesQL.ListMetricsQuery) list
				.getQueryType();
		final EntityReferenceClause entityReferenceClause = listMetrics.getEntityReferenceClause();

		final EntityType[] types = new EntityType[] { EntityType.RESOURCE, EntityType.SERVICE, EntityType.RESOURCE };
		final String[] identfiers = new String[] { "thisIsAnId1", "thisIsAnId2", "thisIsAnId3" };
		final String[] aliases = new String[] { "res1", "svc1", null };

		int i = 0;

		for (final EntityReference ref : entityReferenceClause.getEntityReferences()) {
			assertEquals(types[i], ref.getType());
			assertEquals(identfiers[i], ref.getIdentifier());

			// If we expect a not set alias, test for a null value in the
			// expected value array
			if (aliases[i] != null) {
				assertEquals(aliases[i], ref.getAlias().getAlias());
			} else {
				assertNull(ref.getAlias());
			}

			i++;
		}
	}

}
